@@ -47,6 +47,8 @@ impl StdLib {
     /// (unsafe) [`debug`](https://www.lua.org/manual/5.3/manual.html#6.10) library
     pub const DEBUG: StdLib = StdLib(1 << 31);
 
+    /// No libraries
+    pub const NONE: StdLib = StdLib(0);
     /// (unsafe) All standard libraries
     pub const ALL: StdLib = StdLib(u32::MAX);
     /// The safe subset of the standard libraries
